/**
 EventCell.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit
import SDWebImage
import CoreLocation


class EventCell: UITableViewCell {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    var currentLocation: CLLocation?
    
    var distanceCurrentLocation = 0.0
    
    // Event assigned to one cell
    var event: Event! {
        didSet {
            self.updateUI()
        }
    }
    
    // ===========================================================
    // MARK: - IB Outlets
    // ===========================================================
    
    // Event logo image
    @IBOutlet weak var eventLogoImageView: UIImageView!
    // Event distance label
    @IBOutlet weak var distanceLabel: UILabel!
    
    // ===========================================================
    // MARK: - Methods
    // ===========================================================
    
    /**
     Update UI with event properties
    */
    private func updateUI()
    {
        // Set event properties
        distanceLabel.text = String(format: "%.0f%@", event.distance, "K")
        
        // Download event image
        downloadEventImage()
        
        // Calculate distance between current location
        calculateDistance()
    }
    
    /**
     Download image in background with SDWebImage libraries
    */
    private func downloadEventImage()
    {
        let imageUrl = event.logo?.url
        let url = URL(string: imageUrl!)
        
        // Configuramos un indicador de actividad para que se muestre mientras se carga la imagen
        let activityIndicator = UIActivityIndicatorView()
        activityIndicator.center = CGPoint(x: eventLogoImageView.bounds.midX, y: eventLogoImageView.bounds.midY)
        activityIndicator.hidesWhenStopped = true
        activityIndicator.color = UIColor.black
        eventLogoImageView.addSubview(activityIndicator)
        activityIndicator.startAnimating()
        
        self.eventLogoImageView.sd_setImage(with: url) {(image: UIImage?, error: Error?, cacheType: SDImageCacheType, imageURL: URL?) -> Void in
            if error != nil {
                print(error!)
            } else {
                self.event.logo?.image = image!
            }
            // Al terminar la descarga de la imagen quitamos el indicador de actividad
            activityIndicator.removeFromSuperview()
        }
    }
    
    /**
     Calculate distance between to points
    */
    private func calculateDistance()
    {
        let eventLocation = event.address?.location
        let distance = currentLocation?.distance(from:eventLocation! )
        
        distanceCurrentLocation = distance!.toKilometres()

    }
}
