/**
 EventDetailViewController.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit
import SDWebImage

protocol EventDetailProtocol {
    func showImages(index: Int)
    func showMapDetail()
}

class EventDetailViewController: UIViewController{
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    // Event selected in list
    var detailEvent: Event?
    
    // Controller inside container
    var detailEventController: EventDetailTableViewController?
    
    
    // ===========================================================
    // MARK: - IB Outlets
    // ===========================================================
    
    // Event detail container
    @IBOutlet weak var detailEventContainer: UIView!
    // Event price label
    @IBOutlet weak var priceLabel: UILabel!
    // Event coin label (€, $,...)
    @IBOutlet weak var coinLabel: UILabel!
    @IBOutlet weak var registerView: UIView!
    
    
    // ===========================================================
    // MARK: - IB Actions
    // ===========================================================
    
    @IBAction func unwind(_ segue: UIStoryboardSegue) {}
    
    
    // ===========================================================
    // MARK: - View Lifecycle
    // ===========================================================
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if let height = tabBarController?.tabBar.frame.size.height {
            registerView.heightAnchor.constraint(equalToConstant: height)
        }
        
        updateUI()
        detailEventController?.updateEvent(event: detailEvent!)
        detailEventController?.delegate = self
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        navigationController?.hidesBarsOnSwipe = false
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        tabBarController?.tabBar.isHidden = true
        navigationController?.setNavigationBarHidden(false, animated: true)
    }
    
    
    // ===========================================================
    // MARK: - Methods
    // ===========================================================
    
    /**
     Update UI with event properties
     */
    func updateUI()
    {
        if let event = detailEvent {
            priceLabel.text = event.price.cleanValue
            coinLabel.text = "€"
        }
    }
    
    
    // ===========================================================
    // MARK: - Navigation
    // ===========================================================
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if (segue.identifier == "embedDetail") {
            detailEventController = segue.destination as? EventDetailTableViewController
        }
        
        if (segue.identifier == "segueToMapDetail") {
            if let mapController = segue.destination as? MapViewController {
                mapController.detailEvent = detailEvent
            }
        }
        
        if (segue.identifier == "segueToImageDetail") {
            if let pageImageController = segue.destination as? PageViewController, let index = sender as? Int {
                pageImageController.detailEvent = detailEvent
                pageImageController.currentIndex = index
            }
        }
    }
}


// ===========================================================
// MARK: - EventDetail Protocol
// ===========================================================

extension EventDetailViewController: EventDetailProtocol{
    
    func showMapDetail() {
        performSegue(withIdentifier: "segueToMapDetail", sender: nil)
    }
    
    func showImages(index: Int) {
        performSegue(withIdentifier: "segueToImageDetail", sender: index)
    }
}
