/**
 EventHeaderCell.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit


class EventHeaderCell: UITableViewCell {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    // Event assigned to one cell
    var event: Event! {
        didSet {
            self.updateUI()
        }
    }
    
    // ===========================================================
    // MARK: - IB Outlets
    // ===========================================================
    
    // Event type
    @IBOutlet weak var modeIconImageView: UIImageView!
    // Event name label
    @IBOutlet weak var nameLabel: UILabel!
    // Event date label
    @IBOutlet weak var dateLabel: UILabel!
    // Event location label
    @IBOutlet weak var locationLabel: UILabel!
    // Event price label
    @IBOutlet weak var priceLabel: UILabel!
    // Event coin label (€, $,...)
    @IBOutlet weak var coinLabel: UILabel!
    
    // ===========================================================
    // MARK: - Methods
    // ===========================================================
    
    /**
     Update UI with event properties
     */
    private func updateUI()
    {
        // Set event type image
        modeIconImageView.image = event.type?.image
        
        // Set event properties
        nameLabel.text = event.name
        locationLabel.text = event.address?.toString(format: .short)
        dateLabel.text = event.date.dateStringWith(style: .medium)
        dateLabel.sizeToFit()
        priceLabel.text = event.price.cleanValue
        coinLabel.text = "€"
    }
}
