/**
 ImageViewController.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit
import SDWebImage

class ImageViewController: BaseViewController {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================

    var doubleTap: UITapGestureRecognizer?
    
    var itemIndex: Int = 0
    
    var imageName: String = "" {
        
        didSet {
            
            if (imageView) != nil {
                downloadImage()
            }
            
        }
    }
    
    
    // ===========================================================
    // MARK: - IB Outlets
    // ===========================================================
    
    @IBOutlet weak var scrollView: UIScrollView!
    @IBOutlet var imageView: UIImageView!
    
    
    // ===========================================================
    // MARK: - View Lifecycle
    // ===========================================================

    override func viewDidLoad() {
        super.viewDidLoad()
        
        downloadImage()
        
        self.doubleTap = UITapGestureRecognizer(target: self, action: #selector(ImageViewController.zoom(sender:)))
        self.doubleTap!.numberOfTapsRequired = 2
        self.doubleTap!.numberOfTouchesRequired = 1
        self.scrollView!.addGestureRecognizer(self.doubleTap!)
    }
    
    
    // ===========================================================
    // MARK: - OBJC Responds
    // ===========================================================
    
    @objc func zoom(sender: AnyObject) {
        if let tap = sender as? UITapGestureRecognizer {
            if (self.scrollView!.zoomScale == self.scrollView!.minimumZoomScale) {
                let center = tap.location(in: self.scrollView!)
                let size = self.imageView.bounds.size
                let width = size.width/2
                let height = size.height/2
                let zoomRect = CGRect(x:center.x-width/2, y:center.y-height/2, width:width, height:height)
                self.scrollView!.zoom(to: zoomRect, animated: true)
            } else {
                self.scrollView!.setZoomScale(self.scrollView!.minimumZoomScale, animated: true)
            }
        }
    }
    
    
    // ===========================================================
    // MARK: - Methods
    // ===========================================================
    
    func viewForZooming(in scrollView: UIScrollView) -> UIView? {
        return self.imageView
    }

    func downloadImage() {
        let url = URL(string: imageName)
        
        // Configuramos un indicador de actividad para que se muestre mientras se carga la imagen
        let activityIndicator = UIActivityIndicatorView()
        activityIndicator.center = imageView.center
        activityIndicator.hidesWhenStopped = true
        activityIndicator.color = UIColor.white
        imageView.addSubview(activityIndicator)
        activityIndicator.startAnimating()
        
        // Utilizamos SDWebImage para descargar la imagen en segundo plano directamente sobre el UIImageView de la celda
        imageView.sd_setImage(with: url) {(image: UIImage?, error: Error?, cacheType: SDImageCacheType, imageURL: URL?) -> Void in
            if error != nil {
                print(error!)
            } else {
                //self.eventIma = image!
            }
            // Al terminar la descarga de la imagen quitamos el indicador de actividad
            activityIndicator.removeFromSuperview()
        }
    }

}
