/**
 PageImageViewController.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit

class PageImageViewController: UIPageViewController {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    // Event selected in list
    var detailEvent: Event?
    
    var currentIndex: Int = 0


    // ===========================================================
    // MARK: - View Lifecycle
    // ===========================================================
    
    override func viewDidLoad() {
        super.viewDidLoad()

        self.dataSource = self
        createPageViewController()
        setupPageControl()
    }
    
    
    // ===========================================================
    // MARK: - Methods
    // ===========================================================

    private func createPageViewController() {
        
        if detailEvent!.images.count > 0 {
            let firstController = getItemController(itemIndex: currentIndex)!
            let startingViewControllers = [firstController]
            setViewControllers(startingViewControllers, direction: .forward, animated: false, completion: nil)
        }
    }
    
    private func setupPageControl() {
        let appearance = UIPageControl.appearance()
        appearance.pageIndicatorTintColor = UIColor.white
        appearance.currentPageIndicatorTintColor = UIColor(withHexValue: 0x009688, andAlpha: 1.0)
    }
    
    private func getItemController(itemIndex: Int) -> ImageViewController? {
        
        if itemIndex < detailEvent!.images.count {
            let pageItemController = self.storyboard!.instantiateViewController(withIdentifier: "ItemController") as! ImageViewController
            pageItemController.itemIndex = itemIndex
            pageItemController.imageName = detailEvent!.images[itemIndex].url!
            return pageItemController
        }
        
        return nil
    }
    
    func currentControllerIndex() -> Int {
        
        let pageItemController = self.currentController()
        
        if let controller = pageItemController as? ImageViewController {
            return controller.itemIndex
        }
        
        return -1
    }
    
    func currentController() -> UIViewController? {
        
        if viewControllers!.count > 0 {
            return viewControllers![0]
        }
        
        return nil
    }
}


// ===========================================================
// MARK: - UIPageViewController DataSource
// ===========================================================

extension PageImageViewController: UIPageViewControllerDataSource {
    
    func pageViewController(_ pageViewController: UIPageViewController, viewControllerBefore viewController: UIViewController) -> UIViewController? {
        
        let itemController = viewController as! ImageViewController
        
        if itemController.itemIndex > 0 {
            return getItemController(itemIndex: itemController.itemIndex-1)
        }
        
        return nil
    }
    
    func pageViewController(_ pageViewController: UIPageViewController, viewControllerAfter viewController: UIViewController) -> UIViewController? {
        
        let itemController = viewController as! ImageViewController
        
        if itemController.itemIndex+1 < detailEvent!.images.count {
            return getItemController(itemIndex: itemController.itemIndex+1)
        }
        
        return nil
    }
    
    func presentationCount(for pageViewController: UIPageViewController) -> Int {
        return detailEvent!.images.count
    }
    
    func presentationIndex(for pageViewController: UIPageViewController) -> Int {
        return currentIndex
    }
}
