/**
 SlidesPageViewController.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit

class SlidesPageViewController: UIPageViewController {
  
  // ===========================================================
  // MARK: - Variables and constants
  // ===========================================================
  
  let numSlides = 3
  
  var currentIndex = 0
  
  
  // ===========================================================
  // MARK: - View Lifecycle
  // ===========================================================
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    initPageViewController()
  }
  
  
  // ===========================================================
  // MARK: - Methods
  // ===========================================================
  
  /**
   Init page view controller.
   */
  private func initPageViewController() {
    self.delegate = self
    let firstController = getItemController(by: 0)!
    let startingViewControllers = [firstController]
    setViewControllers(startingViewControllers, direction: .forward, animated: false, completion: nil)
  }
  
  
  /**
   Get view controller by index.
   
   - params itemIndex: Page controller index.
   - returns: Slide view controller
   */
  fileprivate func getItemController(by itemIndex: Int) -> SlideViewController? {
    
    if itemIndex < numSlides {
      let pageItemController = self.storyboard!.instantiateViewController(withIdentifier: "SlideView") as! SlideViewController
      pageItemController.itemIndex = itemIndex
      
      return pageItemController
    }
    
    return nil
  }
  
  /**
   Next page.
   */
  func nextPage() {
    if currentIndex < numSlides {
      currentIndex += 1
      let contentViewControllerToDisplay = getItemController(by: currentIndex)
      setViewControllers([contentViewControllerToDisplay!], direction: .forward, animated: true, completion: nil)
    }
  }
}


// ===========================================================
// MARK: - UIPageViewController Delegate
// ===========================================================

extension SlidesPageViewController: UIPageViewControllerDelegate {
  
  func pageViewController(_ pageViewController: UIPageViewController, didFinishAnimating finished: Bool, previousViewControllers: [UIViewController], transitionCompleted completed: Bool) {
    
    guard completed else { return }
    
    let itemController = pageViewController.viewControllers!.first! as! SlideViewController
    
    currentIndex = itemController.itemIndex
  }
}
