/**
 WelcomeSlidesViewController.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit
import CoreLocation
import UserNotifications

class WelcomeSlidesViewController: BaseViewController {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    // Controller inside container
    var pageController: SlidesPageViewController?
    
    let locationManager = CLLocationManager()
    
    
    // ===========================================================
    // MARK: - IB Outlets
    // ===========================================================
    
    @IBOutlet weak var cancelButton: UIButton!
    @IBOutlet weak var skipButton: UIButton!
    @IBOutlet weak var continueButton: UIButton!
    
    
    // ===========================================================
    // MARK: - IB Actions
    // ===========================================================
    
    @IBAction func skipAction(_ sender: Any) {
        showSignIn()
    }
    
    @IBAction func continueAction(_ sender: Any) {
        
        let numSlides = pageController!.numSlides
        let currentIndex = pageController!.currentIndex
        
        // Permissions view
        if currentIndex == numSlides-2 {
            cancelButton.isHidden = false
        }
        
        if currentIndex < numSlides-1 {
            pageController?.nextPage()
        }
        
        if currentIndex == numSlides-1 {
            checkNotificationsAuthorizationStatus()
        }
    }
    
    @IBAction func cancelAction(_ sender: Any) {
        showSignIn()
    }
    
    // ===========================================================
    // MARK: - View Lifecycle
    // ===========================================================
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Control iPhone/iPad
        if (UIDevice.current.userInterfaceIdiom == .phone) {
            skipButton.titleLabel?.font = UIFont(name: "Avenir-Light", size: 15)
            cancelButton.titleLabel?.font = UIFont(name: "Avenir-Light", size: 15)
            continueButton.titleLabel?.font = UIFont(name: "Avenir-Heavy", size: 20)
        } else {
            skipButton.titleLabel?.font = UIFont(name: "Avenir-Light", size: 25)
            cancelButton.titleLabel?.font = UIFont(name: "Avenir-Light", size: 20)
            continueButton.titleLabel?.font = UIFont(name: "Avenir-Heavy", size: 30)
        }
        
        // Assign text labels
        skipButton.titleLabel?.text = NSLocalizedString("SKIP", comment: "")
        cancelButton.titleLabel?.text = NSLocalizedString("NO_THANKS", comment: "")
        continueButton.titleLabel?.text = NSLocalizedString("CONTINUE", comment: "")
        
        // Assign location manager delegate
        locationManager.delegate = self
        
    }
    
    // ===========================================================
    // MARK: - Permissions
    // ===========================================================
    
    /**
     Check notifications authoritzation status and request it.
     */
    func checkNotificationsAuthorizationStatus() {
        
        UNUserNotificationCenter.current().getNotificationSettings { (notificationSettings) in
            switch notificationSettings.authorizationStatus {
                
            case .notDetermined:
                UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge])
                { granted, error in
                    self.checkLocationAuthorizationStatus()
                }
                
            default:
                break
            }
        }
    }
    
    /**
     Check location authoritzation status and request it.
     */
    func checkLocationAuthorizationStatus() {
        
        
        
        // Ask user authoritzation
        switch CLLocationManager.authorizationStatus() {
            
        case .notDetermined:
            locationManager.requestAlwaysAuthorization()
            
        default:
            break
        }
    }
    
    
    // ===========================================================
    // MARK: - Navigation
    // ===========================================================
    
    func showSignIn() {
        performSegue(withIdentifier: "showSignIn", sender: nil)
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "embedSlidesPage" {
            pageController = segue.destination as? SlidesPageViewController
        }
    }
}


// ===========================================================
// MARK: - CLLocationManager Delegate
// ===========================================================

extension WelcomeSlidesViewController: CLLocationManagerDelegate {
    
    /**
     Change in authorization
     */
    func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {
        
        // Control status
        switch status {
            
        case .notDetermined:
            // Do nothing(when delegate is assigned, it calls this function automatically)
            break
            
        default:
            showSignIn()
        }
    }
}
