/**
 CustomActivityIndicator.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit
import NVActivityIndicatorView

class CustomActivityIndicator: UIVisualEffectView {
    
    var isBlurEffect = false
    
    var text: String? {
        didSet {
            label.text = text
        }
    }
    
    let activityIndictor = NVActivityIndicatorView(frame: .zero, type: .ballTrianglePath, color: UIColor.primary.normal)
    let label: UILabel = UILabel()
    let blurEffect = UIBlurEffect(style: .light)
    let vibrancyView: UIVisualEffectView
    
    init() {
        self.text = ""
        self.vibrancyView = UIVisualEffectView(effect: UIVibrancyEffect(blurEffect: blurEffect))
        super.init(effect: blurEffect)
        self.setup()
    }
    
    required init(coder aDecoder: NSCoder) {
        self.text = ""
        self.vibrancyView = UIVisualEffectView(effect: UIVibrancyEffect(blurEffect: blurEffect))
        super.init(coder: aDecoder)!
        self.setup()
    }
    
    func setup() {
        if isBlurEffect {
            contentView.addSubview(vibrancyView)
            vibrancyView.contentView.addSubview(activityIndictor)
            vibrancyView.contentView.addSubview(label)
        } else {
            contentView.addSubview(activityIndictor)
            contentView.addSubview(label)
        }
    }
    
    override func didMoveToSuperview() {
        super.didMoveToSuperview()
        
        if let superview = self.superview {
            
            // View Size
            let width = superview.frame.size.width / 2
            let height: CGFloat = 50.0
            self.frame = CGRect(x:superview.frame.size.width / 2 - width / 2, y:superview.frame.height / 2 - height / 2, width:width, height:height)
            
            vibrancyView.frame = self.bounds
            
            let activityIndicatorSize: CGFloat = 40
            activityIndictor.frame = CGRect(x:5, y:height / 2 - activityIndicatorSize / 2, width:activityIndicatorSize, height:activityIndicatorSize)
            activityIndictor.color = tintColor
            
            layer.cornerRadius = 8.0
            layer.masksToBounds = true
            label.text = text
            label.textAlignment = NSTextAlignment.center
            label.frame = CGRect(x:activityIndicatorSize + 5, y:0, width:width - activityIndicatorSize - 15, height:height)
            label.textColor = tintColor
            label.font = UIFont(name: "Avenir-Heavy", size: 16)
        }
    }
    
    func show(text: String) {
        self.text = text
        activityIndictor.startAnimating()
        self.isHidden = false
    }
    
    func hide() {
        activityIndictor.stopAnimating()
        self.isHidden = true
    }
}

