/**
 Event.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit
import Firebase
import CoreLocation


class Event {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    private var _eventRef: DatabaseReference
    
    var key: String = ""
    var name: String = ""
    var distance: Float = 0
    var price: Float = 0
    var date: Date = Date()
    var logo: Photo?
    var images: [Photo] = []
    var type: EventType?
    var edition: Int = 0
    var address: Address?
    var phone: String = ""
    var mail: String = ""
    var website: String = ""
    var kml: String = ""
    
    // ===========================================================
    // MARK: - Constructors
    // ===========================================================
    
    /**
     Init with JSON dictionary.
     */
    init(key: String, json: Dictionary<String, AnyObject>) {
        self.key = key
        
        // Within the Event, or Key, the following properties are children
        
        for (elementKey, element) in json {
            
            switch elementKey {
            case "name":
                self.name = element as! String
            case "distance":
                self.distance = element as! Float
            case "price":
                self.price = element as! Float
            case "date":
                self.date = Date(timeIntervalSince1970: element as! TimeInterval)
            case "logo":
                let url = element as! String
                if url != "" {
                    self.logo = Photo(url: url)
                } else {
                    self.logo = Photo(uiImage: #imageLiteral(resourceName: "NoImage"))
                }
            case "images":
                let images = element as! [String]
                for image in images {
                    self.images.append(Photo(url: image))
                }
            case "type":
                self.type = EventType.init(numType: element as! Int)
            case "edition":
                self.edition = element as! Int
            case "phone":
                self.phone = element as! String
            case "mail":
                self.mail = element as! String
            case "website":
                self.website = element as! String
            case "kml":
                self.kml = element as! String
            case "address":
                self.address = Address.init(json: element as! Dictionary<String, AnyObject>)
            default:
                print("No valid JSON element")
            }
        }
        
        // The above properties are assigned to their key.
        
        self._eventRef = FirebaseHelper.sharedInstance.EVENT_REF.child(self.key)
    }
}


