/**
 EventType.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit


struct EventType {
    
    // ===========================================================
    // MARK: - Enumerations
    // ===========================================================

    enum EventTypeEnum {
        case undefined
        case running
        case trailRunning
    }
    
    enum EventCategoryEnum {
        case undefined
        case k5
        case k10
        case halfMarathon
        case marathon
        case ultraMarathon
    }
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    var type: EventTypeEnum
    var category: EventCategoryEnum
    var image: UIImage
    var categoryString: String
    
    // ===========================================================
    // MARK: - Constructors
    // ===========================================================
    
    init(numType: Int) {
        switch numType/10 {
        case 1:
            type = .running
            image = #imageLiteral(resourceName: "Running")
        case 2:
            type = .trailRunning
            image = #imageLiteral(resourceName: "TrailRunning")
        default:
            type = .undefined
            image = #imageLiteral(resourceName: "NoImage")
        }
        switch numType%10 {
        case 1:
            category = .k5
            categoryString = "5K"
        case 2:
            category = .k10
            categoryString = "10K"
        case 3:
            category = .halfMarathon
            categoryString = "Half Marathon"
        case 4:
            category = .marathon
            categoryString = "Marathon"
        case 5:
            category = .ultraMarathon
            categoryString = "Ultra Marathon"
        default:
            category = .undefined
            categoryString = ""
        }
    }
}
