/**
 Photo.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import UIKit


class Photo {
    
    // ===========================================================
    // MARK: - Variables and constants
    // ===========================================================
    
    var url: String?
    var image: UIImage?
    var thumbUrl: String?
    var thumbImage: UIImage?
    
    // ===========================================================
    // MARK: - Constructors
    // ===========================================================
    
    // Init from UIImage
    init(uiImage: UIImage) {
        self.image = uiImage
        self.thumbImage = createThumbnail(image: uiImage)
    }
    
    // Init from url
    init(url: String) {
        self.url = url
    }
    
    // ===========================================================
    // MARK: - Methods
    // ===========================================================
    
    /**
     Create a reduced image
     
     - parameter image: Original image
     
     - returns: Reduced image
     */
    func createThumbnail(image: UIImage) -> UIImage? {
        let maxWidth: Double = 200
        let maxHeight: Double = 200
        
        let imageWidth = Double(image.size.width)
        let imageHeight = Double(image.size.height)
        
        var scaleFactor: Double
        
        if imageWidth > imageHeight {
            scaleFactor = maxWidth / imageWidth
        } else {
            scaleFactor = maxHeight / imageHeight
        }
        
        let thumbWidth = imageWidth * scaleFactor
        let thumbHeight = imageHeight * scaleFactor
        
        let size = CGSize(width: thumbWidth, height: thumbHeight)
        
        let thumbImage = image.resizeImage(size: size)
        
        return thumbImage
    }
}
