/**
 FacebookAuth.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import FBSDKLoginKit


/**
 Facebook authoritzation utilities.
 */
class FacebookAuth {
  
  /**
   Check if user is login.
   
   - returns: User is login.
   */
  static func isLogin() -> Bool {
    return FBSDKAccessToken.current() != nil
  }
  
  /**
   Sign in.
   */
  static func signIn(viewController: UIViewController, completion: @escaping (_ result: FBSDKLoginManagerLoginResult?, _ error: Error?) -> Void) {
    let login = FBSDKLoginManager.init()
    
    login.logIn(withReadPermissions: ["public_profile", "email"], from: viewController) { result,error in
      completion(result,error)
    }
  }
  
  /**
   Sign out.
   */
  static func signOut() {
    FBSDKLoginManager().logOut()
  }
  
  /**
   Check if user is login.
   
   - parameter url: Check if url is a facebook valid url.
   
   - returns: Valid url.
   */
  static func isValidatedWithUrl(url: URL) -> Bool {
    return url.scheme!.hasPrefix("fb\(FBSDKSettings.appID()!)") && url.host! == "authorize"
  }
}

