/**
 GoogleAuth.swift
 
 This file is part of the Eventhon package.
 @copyright (c) 2017-present, Eventhon. All rights reserved.
 @author Raul Romero Rostan <rromerorostan@gmail.com>
 @updated 2017-11-02
 
 For the full copyright and license information, please view the LICENSE
 file that was distributed with this source code.
 */

import GoogleSignIn


/**
 Google authoritzation utilities.
 */
class GoogleAuth {
  
  /**
   Sigleton pattern.
   
   - returns: Google instance.
   */
  static func getInstance() -> GIDSignIn {
    return GIDSignIn.sharedInstance()
  }
  
  /**
   Check if user is login.
   
   - returns: User is login.
   */
  static func isLogin() -> Bool {
    return getInstance().hasAuthInKeychain()
  }
  
  /**
   Sign in.
   */
  static func signIn() {
    getInstance().signIn()
  }
  
  /**
   Sign out.
   */
  static func signOut() {
    getInstance().signOut()
  }
  
  /**
   Check if user is login.
   
   - parameter url: Check if url is a google valid url.
   
   - returns: Valid url.
   */
  static func isValidatedWithUrl(url: URL) -> Bool {
    return url.scheme!.hasPrefix(Bundle.main.bundleIdentifier!) || url.scheme!.hasPrefix("com.googleusercontent.apps.")
  }
}

